@@ -22,17 +22,25 @@
 #include "trace-kernel.h"
 
 int kernel_consumer_send_channel_stream(struct consumer_socket *sock,
-		struct ltt_kernel_channel *channel, struct ltt_kernel_session *session);
+		struct ltt_kernel_channel *channel, struct ltt_kernel_session *session,
+		unsigned int monitor);
 
 int kernel_consumer_send_session(struct consumer_socket *sock,
 		struct ltt_kernel_session *session);
 
 int kernel_consumer_add_stream(struct consumer_socket *sock,
 		struct ltt_kernel_channel *channel, struct ltt_kernel_stream *stream,
-		struct ltt_kernel_session *session);
+		struct ltt_kernel_session *session, unsigned int monitor);
 
 int kernel_consumer_add_metadata(struct consumer_socket *sock,
-		struct ltt_kernel_session *session);
+		struct ltt_kernel_session *session, unsigned int monitor);
 
 int kernel_consumer_add_channel(struct consumer_socket *sock,
+		struct ltt_kernel_channel *channel, struct ltt_kernel_session *session,
+		unsigned int monitor);
+
+int kernel_consumer_destroy_channel(struct consumer_socket *socket,
 		struct ltt_kernel_channel *channel);
+
+int kernel_consumer_destroy_metadata(struct consumer_socket *socket,
+		struct ltt_kernel_metadata *metadata);
