@@ -23,6 +23,9 @@
 #include <common/error.h>
 
 #include "utils.h"
+#include "lttng-sessiond.h"
+
+int ht_cleanup_pipe[2] = { -1, -1 };
 
 /*
  * Write to writable pipe used to notify a thread.
@@ -46,12 +49,26 @@ int notify_thread_pipe(int wpipe)
 	return ret;
 }
 
-/*
- * Return pointer to home directory path using the env variable HOME.
- *
- * No home, NULL is returned.
- */
-const char *get_home_dir(void)
+void ht_cleanup_push(struct lttng_ht *ht)
 {
-	return ((const char *) getenv("HOME"));
+	int ret;
+	int fd = ht_cleanup_pipe[1];
+
+	if (fd < 0)
+		return;
+	do {
+		ret = write(fd, &ht, sizeof(ht));
+	} while (ret < 0 && errno == EINTR);
+	if (ret < 0 || ret != sizeof(ht)) {
+		PERROR("write ht cleanup pipe %d", fd);
+		if (ret < 0) {
+			ret = -errno;
+		}
+		goto error;
+	}
+
+	/* All good. Don't send back the write positive ret value. */
+	ret = 0;
+error:
+	assert(!ret);
 }
