@@ -71,7 +71,7 @@ static struct poptOption long_options[] = {
  */
 static void usage(FILE *ofp)
 {
-	fprintf(ofp, "usage: lttng list [OPTIONS] [SESSION [<OPTIONS>]]\n");
+	fprintf(ofp, "usage: lttng list [OPTIONS] [SESSION [SESSION OPTIONS]]\n");
 	fprintf(ofp, "\n");
 	fprintf(ofp, "With no arguments, list available tracing session(s)\n");
 	fprintf(ofp, "\n");
@@ -128,13 +128,23 @@ static
 const char *active_string(int value)
 {
 	switch (value) {
-	case 0:	return " [inactive]";
-	case 1: return " [active]";
+	case 0:	return "inactive";
+	case 1: return "active";
 	case -1: return "";
 	default: return NULL;
 	}
 }
 
+static const char *snapshot_string(int value)
+{
+	switch (value) {
+	case 1:
+		return " snapshot";
+	default:
+		return "";
+	}
+}
+
 static
 const char *enabled_string(int value)
 {
@@ -220,6 +230,17 @@ static void print_events(struct lttng_event *event)
 		}
 		break;
 	}
+	case LTTNG_EVENT_FUNCTION:
+		MSG("%s%s (type: function)%s%s", indent6,
+				event->name, enabled_string(event->enabled),
+				filter_string(event->filter));
+		if (event->attr.probe.addr != 0) {
+			MSG("%saddr: 0x%" PRIx64, indent8, event->attr.probe.addr);
+		} else {
+			MSG("%soffset: 0x%" PRIx64, indent8, event->attr.probe.offset);
+			MSG("%ssymbol: %s", indent8, event->attr.probe.symbol_name);
+		}
+		break;
 	case LTTNG_EVENT_PROBE:
 		MSG("%s%s (type: probe)%s%s", indent6,
 				event->name, enabled_string(event->enabled),
@@ -231,7 +252,6 @@ static void print_events(struct lttng_event *event)
 			MSG("%ssymbol: %s", indent8, event->attr.probe.symbol_name);
 		}
 		break;
-	case LTTNG_EVENT_FUNCTION:
 	case LTTNG_EVENT_FUNCTION_ENTRY:
 		MSG("%s%s (type: function)%s%s", indent6,
 				event->name, enabled_string(event->enabled),
@@ -294,6 +314,7 @@ static int list_ust_events(void)
 	struct lttng_handle *handle;
 	struct lttng_event *event_list;
 	pid_t cur_pid = 0;
+	char *cmdline = NULL;
 
 	memset(&domain, 0, sizeof(domain));
 
@@ -308,7 +329,7 @@ static int list_ust_events(void)
 
 	size = lttng_list_tracepoints(handle, &event_list);
 	if (size < 0) {
-		ERR("Unable to list UST events");
+		ERR("Unable to list UST events: %s", lttng_strerror(size));
 		lttng_destroy_handle(handle);
 		return size;
 	}
@@ -322,7 +343,9 @@ static int list_ust_events(void)
 	for (i = 0; i < size; i++) {
 		if (cur_pid != event_list[i].pid) {
 			cur_pid = event_list[i].pid;
-			MSG("\nPID: %d - Name: %s", cur_pid, get_cmdline_by_pid(cur_pid));
+			cmdline = get_cmdline_by_pid(cur_pid);
+			MSG("\nPID: %d - Name: %s", cur_pid, cmdline);
+			free(cmdline);
 		}
 		print_events(&event_list[i]);
 	}
@@ -349,6 +372,8 @@ static int list_ust_event_fields(void)
 	struct lttng_handle *handle;
 	struct lttng_event_field *event_field_list;
 	pid_t cur_pid = 0;
+	char *cmdline = NULL;
+
 	struct lttng_event cur_event;
 
 	memset(&domain, 0, sizeof(domain));
@@ -365,7 +390,7 @@ static int list_ust_event_fields(void)
 
 	size = lttng_list_tracepoint_fields(handle, &event_field_list);
 	if (size < 0) {
-		ERR("Unable to list UST event fields");
+		ERR("Unable to list UST event fields: %s", lttng_strerror(size));
 		lttng_destroy_handle(handle);
 		return size;
 	}
@@ -379,7 +404,9 @@ static int list_ust_event_fields(void)
 	for (i = 0; i < size; i++) {
 		if (cur_pid != event_field_list[i].event.pid) {
 			cur_pid = event_field_list[i].event.pid;
-			MSG("\nPID: %d - Name: %s", cur_pid, get_cmdline_by_pid(cur_pid));
+			cmdline = get_cmdline_by_pid(cur_pid);
+			MSG("\nPID: %d - Name: %s", cur_pid, cmdline);
+			free(cmdline);
 		}
 		if (strcmp(cur_event.name, event_field_list[i].event.name) != 0) {
 			print_events(&event_field_list[i].event);
@@ -424,7 +451,7 @@ static int list_kernel_events(void)
 
 	size = lttng_list_tracepoints(handle, &event_list);
 	if (size < 0) {
-		ERR("Unable to list kernel events");
+		ERR("Unable to list kernel events: %s", lttng_strerror(size));
 		lttng_destroy_handle(handle);
 		return size;
 	}
@@ -458,6 +485,7 @@ static int list_events(const char *channel_name)
 	count = lttng_list_events(handle, channel_name, &events);
 	if (count < 0) {
 		ret = count;
+		ERR("%s", lttng_strerror(ret));
 		goto error;
 	}
 
@@ -474,9 +502,7 @@ static int list_events(const char *channel_name)
 	MSG("");
 
 end:
-	if (events) {
-		free(events);
-	}
+	free(events);
 	ret = CMD_SUCCESS;
 
 error:
@@ -530,6 +556,7 @@ static int list_channels(const char *channel_name)
 			/* We had a real error */
 			ret = count;
 			ERR("%s", lttng_strerror(ret));
+			break;
 		}
 		goto error_channels;
 	}
@@ -551,7 +578,7 @@ static int list_channels(const char *channel_name)
 		/* Listing events per channel */
 		ret = list_events(channels[i].name);
 		if (ret < 0) {
-			MSG("%s", lttng_strerror(ret));
+			ERR("%s", lttng_strerror(ret));
 		}
 
 		if (chan_found) {
@@ -588,6 +615,7 @@ static int list_sessions(const char *session_name)
 	DBG("Session count %d", count);
 	if (count < 0) {
 		ret = count;
+		ERR("%s", lttng_strerror(ret));
 		goto error;
 	} else if (count == 0) {
 		MSG("Currently no available tracing session");
@@ -602,18 +630,16 @@ static int list_sessions(const char *session_name)
 		if (session_name != NULL) {
 			if (strncmp(sessions[i].name, session_name, NAME_MAX) == 0) {
 				session_found = 1;
-				MSG("Tracing session %s:%s", session_name, active_string(sessions[i].enabled));
+				MSG("Tracing session %s: [%s%s]", session_name,
+						active_string(sessions[i].enabled),
+						snapshot_string(sessions[i].snapshot_mode));
 				MSG("%sTrace path: %s\n", indent4, sessions[i].path);
 				break;
 			}
-			continue;
-		}
-
-		MSG("  %d) %s (%s)%s", i + 1, sessions[i].name, sessions[i].path,
-				active_string(sessions[i].enabled));
-
-		if (session_found) {
-			break;
+		} else {
+			MSG("  %d) %s (%s) [%s%s]", i + 1, sessions[i].name, sessions[i].path,
+					active_string(sessions[i].enabled),
+					snapshot_string(sessions[i].snapshot_mode));
 		}
 	}
 
@@ -649,6 +675,7 @@ static int list_domains(const char *session_name)
 	count = lttng_list_domains(session_name, &domains);
 	if (count < 0) {
 		ret = count;
+		ERR("%s", lttng_strerror(ret));
 		goto error;
 	} else if (count == 0) {
 		MSG("  None");
@@ -785,6 +812,7 @@ int cmd_list(int argc, const char **argv)
 			nb_domain = lttng_list_domains(session_name, &domains);
 			if (nb_domain < 0) {
 				ret = nb_domain;
+				ERR("%s", lttng_strerror(ret));
 				goto end;
 			}
 
@@ -795,6 +823,9 @@ int cmd_list(int argc, const char **argv)
 					break;
 				case LTTNG_DOMAIN_UST:
 					MSG("=== Domain: UST global ===\n");
+					MSG("Buffer type: %s\n",
+							domains[i].buf_type ==
+							LTTNG_BUFFER_PER_PID ? "per PID" : "per UID");
 					break;
 				default:
 					MSG("=== Domain: Unimplemented ===\n");
@@ -821,9 +852,7 @@ int cmd_list(int argc, const char **argv)
 	}
 
 end:
-	if (domains) {
-		free(domains);
-	}
+	free(domains);
 	if (handle) {
 		lttng_destroy_handle(handle);
 	}
