@@ -26,6 +26,8 @@
  * Include this file _after_ including your URCU flavor.
  */
 
+#include "hashtable-symbols.h"
+
 #include <stdint.h>
 #include <urcu/compiler.h>
 #include <urcu-call-rcu.h>
@@ -176,6 +178,8 @@ struct cds_lfht *cds_lfht_new(unsigned long init_size,
  *
  * Return 0 on success, negative error value on error.
  * Threads calling this API need to be registered RCU read-side threads.
+ * cds_lfht_destroy should *not* be called from a RCU read-side critical
+ * section.
  */
 int cds_lfht_destroy(struct cds_lfht *ht, pthread_attr_t **attr);
 
@@ -425,6 +429,8 @@ int cds_lfht_is_node_deleted(struct cds_lfht_node *node);
  *
  * Threads calling this API need to be registered RCU read-side threads.
  * This function does not (necessarily) issue memory barriers.
+ * cds_lfht_resize should *not* be called from a RCU read-side critical
+ * section.
  */
 void cds_lfht_resize(struct cds_lfht *ht, unsigned long new_size);
 
